version 12
clear all

cd "C:\ado\personal"	


capture program drop vx_dbel
	
program vx_dbel, rclass
	version 12
	syntax varlist  [, repsnum(real 10000) sided(string) method(string)]
	
capture mata: mata drop vx_dbel.invreg()
	preserve
  
	quietly putmata datmatx=(`varlist'), replace
	
	local state = c(seed)
	set seed `state'
	
	///////////////////////// method monte carlo ///////////////////////
	if (strmatch(strlower("`method'"),"mc") )        mata: dbelmc(datmatx,`repsnum',"`varlist'","`sided'")
	
	/////////////////////////// method table ///////////////////////////
	//// one sample
	if (strmatch(strlower("`method'"),"table") )   	{
						mata: ncols=cols(datmatx)
						mata: nrows=nrows(datmatx)
						mata: reg=reg(datmatx)
						
						use critval1s, clear
						quietly putmata crit1s=(*), replace omitmissing
						mata: ts=dbeltable(reg, nrows, crit1s, datmatx, "`varlist'", "`sided'")
						
						mata: decision=tableexact(reg, nrows, ncols, ts, crit1s, "`sided'")
												
						drop _all
						getmata (reg)=reg, replace force
						local regflag=reg 
						
						if (`regflag'==1) {
									drop _all
									getmata (sided alpha n cval reg predict)=crit1s, replace force
		
									gen invn=1/n
									gen n2=n^2
									gen alphahalf=alpha^0.5
									qui reg cval n invn n2 alpha alphahalf
									
									mata: betas = st_matrix("e(b)")
									mata: decision=getroot(crit1s, betas, ts, "`sided'")
									
								}	
						quietly drop _all
						getmata (decision)=decision, replace force
						return scalar reject=decision
			}
end


mata:
	void function getoptim(todo,b,c1,c2,c3,crit,g,H)
	{
		crit=(c1+c2*b+c3*b^0.5)^2
	}
end


mata: 
	function invreg(x,c1,c2,c3) return(c1+c2*x+c3*x^0.5)
end

mata: 
	real scalar function getroot(real matrix crit1s, real rowvector betas, real scalar ts, string sided)
		{ 
			real scalar c1, c2, c3, testroot, rc, pvalue, max_cval, min_cval
			
			pvalue=9.9999 
			
			npred=crit1s[1,6]
			max_cval=colmax(crit1s[.,4])
			min_cval=colmin(crit1s[.,4])
			
			c1=betas[1,6] + betas[1,1]*npred + betas[1,2]*(1/npred)+ betas[1,3]*(npred^2) - ts
			c2=betas[1,4]
			c3=betas[1,5]
			
			testroot=invreg(0,c1,c2,c3)*invreg(1,c1,c2,c3)

			if (testroot<0) rc=mm_root(pvalue=.,&invreg(),0,1,0,10000,c1,c2,c3)	
			
			if (testroot>=0 & ts<=max_cval & ts>=min_cval) {
					S=optimize_init()
					optimize_init_argument(S, 1, c1)
					optimize_init_argument(S, 2,  c2)
					optimize_init_argument(S, 3, c3)
					optimize_init_evaluator(S, &getoptim())
					optimize_init_which(S,"min") 
					optimize_init_params(S,0.9)
					optimize_init_nmsimplexdeltas(S, (10e-4))
					optimize_init_technique(S, "nm")
   
					pvalue=round(optimize(S),10e-4)
				}
				
			//pvalue, ts, min_cval, max_cval
			if (ts>max_cval)  pvalue=0
			if (ts<min_cval)  pvalue=1
			//pvalue
			
			if (pvalue<0.05) 	decision=1
			if (pvalue>=0.05)   decision=0
			//testroot, decision, pvalue
			
			return(decision)
		}
end



/////////////////////One sided one sample///////////////////////////

mata:
	real scalar function wwsim1s(real scalar xn, real scalar yn)
		{
			real colvector x, y 
			real scalar log_vnk
			
			x = sort(rnormal(xn,1,0,1),1)
			y = sort(-x,1)
			
			log_vnk=log(elr1sw(x,y))
			//log_vnk
			
			return(log_vnk)
		}
end


mata:
	real scalar function wwt01s(real colvector x, real colvector y)
		{
			real scalar xn, yn, teststat
			
			xn=rows(x)
			yn=rows(y)
						
			x=sort(x,1)
			y=sort(y,1)
			
			teststat=log(elr1sw(x,y))
			//teststat
			
			return(teststat)
		}
end


mata:
	real scalar function elr1sw(real colvector q1, real colvector qq)
		{
		
			real colvector m, rm, mm, L, LL, U, UU, qL, qU, F, Fnum, Fnum3
			real rowvector temp, prodFmat
			real matrix Fmat
			real scalar q1n, qqn, elr, Fnum1, Fnum2
			
			q1n=rows(q1)
			qqn=rows(qq)
			invqt=1/(q1n)

			m=(round(q1n^0.6)::min(( round(q1n^.9),round(q1n/2) )))

			rm=m#J(q1n,1,1)
			mm=J(rows(m),1,1)#(1::q1n)

			L=mm-rm
			LL=(L:*(L:>0))+(L:<=0)
			U=mm+rm
			UU=(U:*(U:<=q1n))+(U:>q1n)*q1n

			qL=q1[LL]
			qU=q1[UU]
			
		    F=rowmax((mm_relrank(q1,q1n,qU),mm_relrank(qq,qqn,qU)))-rowmax((mm_relrank(q1,q1n,qL),mm_relrank(qq,qqn,qL)))
			
			F=editvalue(F,0,invqt)
			
			Fnum1=2*rm
			Fnum2=1/(2*q1n)
			Fnum3=rm:+1
			Fnum=Fnum1:*(1:-(Fnum2*Fnum3))
			
			F=Fnum:/(q1n*F)
			Fmat=colshape(F,q1n)
			_transpose(Fmat)
			
			prodFmat=exp(colsum(log(Fmat)))
		
			elr=min(prodFmat)
	
			return(elr)
		}
end

////////////////////// two sided one sample //////////////////////

mata:
	real scalar function wwsim(real scalar xn, real scalar yn)
		{
			real colvector x, y, z 
			real scalar log_vnk
			
			x = sort(rnormal(xn,1,0,1),1)
			y = sort(-x,1)
			z = sort((x\y),1)	
			
			log_vnk=log(elrw(x,z))
			//log_vnk
			
			return(log_vnk)
		}
end


mata:
	real scalar function wwt0(real colvector x, real colvector y)
		{
			real colvector z
			real scalar xn, yn, teststat
			
			xn=rows(x)
			yn=rows(y)
						
			x=sort(x,1)
			y=sort(y,1)
			z=sort((x\y),1)
			
			teststat=log(elrw(x,z))
			//teststat
			
			return(teststat)
		}
end


mata:
	real scalar function elrw(real colvector q1, real colvector qq)
		{
			real colvector m, rm, mm, L, LL, U, UU, qL, qU, F, Fnum, Fnum3
			real rowvector temp, prodFmat
			real matrix Fmat
			real scalar q1n, qqn, elr, Fnum1, Fnum2
			
			q1n=rows(q1)
			qqn=rows(qq)
			invqt=1/(q1n)

			m=(round(q1n^0.6)::min(( round(q1n^.9),round(q1n/2) )))

			rm=m#J(q1n,1,1)
			mm=J(rows(m),1,1)#(1::q1n)

			L=mm-rm
			LL=(L:*(L:>0))+(L:<=0)
			U=mm+rm
			UU=(U:*(U:<=q1n))+(U:>q1n)*q1n

			qL=q1[LL]
			qU=q1[UU]
			
			F=mm_relrank(qq,rows(qq),qU)-mm_relrank(qq,rows(qq),qL)
			F=editvalue(F,0,1/(qqn))

			Fnum1=2*rm
			Fnum2=1/(2*q1n)
			Fnum3=rm:+1
			Fnum=Fnum1:*(1:-(Fnum2*Fnum3))
			
			F=Fnum:/(q1n*F)
						
			Fmat=colshape(F,q1n)
			_transpose(Fmat)
				
			prodFmat=exp(colsum(log(Fmat)))
			
			elr=min(prodFmat)
		
			return(elr)
		}
end



////////////////////////// begin and branch by test ////////////////////////

//////////////// mc /////////////////
mata:
	void dbelmc(real matrix x, real scalar rnum, string dsnlist, string testside)
	{
		
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
		
		mc=J(rnum,1,0)
		
		//////// two sided one sample ////////
		if (strmatch(testside,"one.sample"))
			{
				nn=1
				yy=-xx
				while(nn<=rnum) {   
					  mc[nn,1]=wwsim(rows(xx),rows(yy))
					  ++nn
					}
				mc=sort(mc,1)
				t0=wwt0(xx,yy)
				pvalue=(colsum(mc:>=t0))/rnum
				
				test="DBEL (1-sample, 2-sided)"
				
				printf("{txt}{space 25}{c |}  T0               P-value\n")
				printf("{hline 25}{c +}{hline 34}\n")
				printf("{txt}%24s {c |} {res}%10.0g %10.0g\n",test, t0, pvalue)
				printf("{txt}{space 25}\n\n\n")
			}
			
		//////// one sided one sample ////////	
		if (strmatch(testside,"greater") & cols(x)==1)
			{
				nn=1
				yy=-xx
				while(nn<=rnum) {   
					  mc[nn,1]=wwsim1s(rows(xx),rows(yy))
					  ++nn
					}
				mc=sort(mc,1)
				t0=wwt01s(xx,yy)
				pvalue=(colsum(mc:>=t0))/rnum
				
				test="DBEL (1-sample, 1-sided)"
				
				printf("{txt}{space 25}{c |}  T0               P-value\n")
				printf("{hline 25}{c +}{hline 34}\n")
				printf("{txt}%24s {c |} {res}%10.0g %10.0g\n",test, t0, pvalue)
				printf("{txt}{space 25}\n\n\n")
			}	
	}
		
end

////////////// table /////////////////
mata:
real scalar reg( real matrix x)
	{
		nrows=J(1,3,0)
		reg=J(1,1,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		if (nrows[1,1]>30 | nrows[1,2]>30 | nrows[1,3]>30 )  reg=1
		
		return(reg)
	}		
end

mata:
real rowvector nrows( real matrix x)
	{
		nrows=J(1,3,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		nrows=nrows[1,1],nrows[1,2],nrows[1,3]
	
		return(nrows)
	}		
end

mata:
real scalar dbeltable(real scalar reg, real rowvector nrows, real matrix crit1s, real matrix x, string dsnlist, string testside)
	{
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
		
		//crit1s
				
		if (strmatch(testside,"greater") & cols(x)==1)
			{
				crit1s=select(crit1s[.,1::13],crit1s[.,1]:==1)
				yy=-xx
				t0=wwt01s(xx,yy)
				if (reg==1) {
								crit1s=select(crit1s[.,1::13],crit1s[.,3]:>=30)
								if (30<nrows[1,1]  & nrows[1,1]<35) 	crit1s=select(crit1s[.,1::4],crit1s[.,5]:==1)
								if (35<=nrows[1,1] & nrows[1,1]<40)		crit1s=select(crit1s[.,1::4],crit1s[.,6]:==1)
								if (40<=nrows[1,1] & nrows[1,1]<50) 	crit1s=select(crit1s[.,1::4],crit1s[.,7]:==1)
								if (50<=nrows[1,1] & nrows[1,1]<60) 	crit1s=select(crit1s[.,1::4],crit1s[.,8]:==1)
								if (60<=nrows[1,1] & nrows[1,1]<80) 	crit1s=select(crit1s[.,1::4],crit1s[.,9]:==1)
								if (80<=nrows[1,1] & nrows[1,1]<100) 	crit1s=select(crit1s[.,1::4],crit1s[.,10]:==1)
								if (100<=nrows[1,1] & nrows[1,1]<150) 	crit1s=select(crit1s[.,1::4],crit1s[.,11]:==1)
								if (150<=nrows[1,1] & nrows[1,1]<200) 	crit1s=select(crit1s[.,1::4],crit1s[.,12]:==1)
								if (200<=nrows[1,1]) 					crit1s=select(crit1s[.,1::4],crit1s[.,13]:==1)		
							
								addinfo=J(rows(crit1s),2,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								crit1s=(crit1s,addinfo)
								
							}
				if (reg==0) crit1s=select(crit1s[.,1::4],crit1s[.,3]:==nrows[1,1])
				
			}
			
		if (strmatch(testside,"one.sample"))
			{
				crit1s=select(crit1s[.,1::13],crit1s[.,1]:==2)
				yy=-xx
				t0=wwt0(xx,yy)
				if (reg==1) {
								crit1s=select(crit1s[.,1::13],crit1s[.,3]:>=30)
								if (30<nrows[1,1]  & nrows[1,1]<35) 	crit1s=select(crit1s[.,1::4],crit1s[.,5]:==1)
								if (35<=nrows[1,1] & nrows[1,1]<40)		crit1s=select(crit1s[.,1::4],crit1s[.,6]:==1)
								if (40<=nrows[1,1] & nrows[1,1]<50) 	crit1s=select(crit1s[.,1::4],crit1s[.,7]:==1)
								if (50<=nrows[1,1] & nrows[1,1]<60) 	crit1s=select(crit1s[.,1::4],crit1s[.,8]:==1)
								if (60<=nrows[1,1] & nrows[1,1]<80) 	crit1s=select(crit1s[.,1::4],crit1s[.,9]:==1)
								if (80<=nrows[1,1] & nrows[1,1]<100) 	crit1s=select(crit1s[.,1::4],crit1s[.,10]:==1)
								if (100<=nrows[1,1] & nrows[1,1]<150) 	crit1s=select(crit1s[.,1::4],crit1s[.,11]:==1)
								if (150<=nrows[1,1] & nrows[1,1]<200) 	crit1s=select(crit1s[.,1::4],crit1s[.,12]:==1)
								if (200<=nrows[1,1]) 					crit1s=select(crit1s[.,1::4],crit1s[.,13]:==1)		
								
								addinfo=J(rows(crit1s),2,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								crit1s=(crit1s,addinfo)
								
							}
				if (reg==0) crit1s=select(crit1s[.,1::4],crit1s[.,3]:==nrows[1,1])			
	
			}
	
		return(t0)
	}
		
end


mata:

real scalar tableexact(real scalar reg,real rowvector nrows, real scalar ncols, real scalar ts, real matrix crit1s, string testside)
	{
		real scalar decision
		decision=0
		//decision
		//////// one sample -- 1s & 2s  ////////
		if (ncols==1 & reg==0)
			{
				Tcrit1s=crit1s'
				//Tcrit1s
				
				if (ts>Tcrit1s[4,5]) 	decision=1
				if (ts<=Tcrit1s[4,5]) 	decision=0
				//ts
				//Tcrit1s
				//decision		
			}
		return(decision)
	}
		
end
	
//vx_dbel dsn1, sided("greater") method("table")
//vx_dbel dsn1, sided("greater") method("mc")

//vx_dbel dsn1 , sided("one.sample") method("table")
//vx_dbel dsn1, sided("one.sample") method("mc")

//////////////////////////////////////////////////////////


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(5, 1, 0, 1)
	//mata: dsn1 = rnormal(5, 1, 0.5, 1)
	//mata: dsn1 = rnormal(5, 1, 0.25, 1)
	//mata: dsn1 = runiform(5,1):-0.3
	mata: dsn1 = (runiform(5,1):*1.8):-0.8
	//mata: dsn1 = (runiform(5,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw1a=colsum(result)/10000
mata: powerw1a	
mata: select(powerw1a,powerw1a:>1)	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(10, 1, 0, 1)
	//mata: dsn1 = rnormal(10, 1, 0.5, 1)
	//mata: dsn1 = rnormal(10, 1, 0.25, 1)
	//mata: dsn1 = runiform(10,1):-0.3
	mata: dsn1 = (runiform(10,1):*1.8):-0.8
	//mata: dsn1 = (runiform(10,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw1b=colsum(result)/10000
mata: powerw1b	
mata: select(powerw1b,powerw1b:>1)	



mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(15, 1, 0, 1)
	//mata: dsn1 = rnormal(15, 1, 0.5, 1)
	//mata: dsn1 = rnormal(15, 1, 0.25, 1)
	//mata: dsn1 = runiform(15,1):-0.3
	mata: dsn1 = (runiform(15,1):*1.8):-0.8
	//mata: dsn1 = (runiform(15,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw1c=colsum(result)/10000
mata: powerw1c	
mata: select(powerw1c,powerw1c:>1)	




mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(20, 1, 0, 1)
	//mata: dsn1 = rnormal(20, 1, 0.5, 1)
	//mata: dsn1 = rnormal(20, 1, 0.25, 1)
	//mata: dsn1 = runiform(20,1):-0.3
	mata: dsn1 = (runiform(20,1):*1.8):-0.8
	//mata: dsn1 = (runiform(20,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw1d=colsum(result)/10000
mata: powerw1d	
mata: select(powerw1d,powerw1d:>1)	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(25, 1, 0, 1)
	//mata: dsn1 = rnormal(25, 1, 0.5, 1)
	//mata: dsn1 = rnormal(25, 1, 0.25, 1)
	//mata: dsn1 = runiform(25,1):-0.3
	mata: dsn1 = (runiform(25,1):*1.8):-0.8
	//mata: dsn1 = (runiform(25,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw1e=colsum(result)/10000
mata: powerw1e	
mata: select(powerw1e,powerw1e:>1)	



mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(30, 1, 0, 1)
	//mata: dsn1 = rnormal(30, 1, 0.5, 1)
	//mata: dsn1 = rnormal(30, 1, 0.25, 1)
	//mata: dsn1 = runiform(30,1):-0.3
	mata: dsn1 = (runiform(30,1):*1.8):-0.8
	//mata: dsn1 = (runiform(30,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw1f=colsum(result)/10000
mata: powerw1f	
mata: select(powerw1f,powerw1f:>1)	



mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(35, 1, 0, 1)
	//mata: dsn1 = rnormal(35, 1, 0.5, 1)
	//mata: dsn1 = rnormal(35, 1, 0.25, 1)
	//mata: dsn1 = runiform(35,1):-0.3
	mata: dsn1 = (runiform(35,1):*1.8):-0.8
	//mata: dsn1 = (runiform(35,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	qui vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw1=colsum(result)/10000
mata: powerw1	
mata: select(powerw1,powerw1:>1)	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(40, 1, 0, 1)
	//mata: dsn1 = rnormal(40, 1, 0.5, 1)
	//mata: dsn1 = rnormal(40, 1, 0.25, 1)
	//mata: dsn1 = runiform(40,1):-0.3
	mata: dsn1 = (runiform(40,1):*1.8):-0.8
	//mata: dsn1 = (runiform(40,1):*2):-1
	
	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	qui vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw2=colsum(result)/10000
mata: powerw2	
mata: select(powerw2,powerw2:>1)	



mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate data to analyze
	quietly drop _all
	//mata: dsn1 = rnormal(45, 1, 0, 1)
	//mata: dsn1 = rnormal(45, 1, 0.5, 1)
	//mata: dsn1 = rnormal(45, 1, 0.25, 1)
	//mata: dsn1 = runiform(45,1):-0.3
	mata: dsn1 = (runiform(45,1):*1.8):-0.8
	//mata: dsn1 = (runiform(45,1):*2):-1

	getmata dsn1 
	
	//vx_dbel dsn1, sided("one.sample") method("table")
	qui vx_dbel dsn1, sided("greater") method("table")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}
			
mata: powerw3=colsum(result)/10000
mata: powerw3	
mata: select(powerw3,powerw3:>1)	



	
	

//mata: mata describe
mata: powerw1a
mata: powerw1b
mata: powerw1c
mata: powerw1d
mata: powerw1e
mata: powerw1f
mata: powerw1
mata: powerw2
mata: powerw3

