version 12
clear all

cd "C:\ado\personal"	


/////////////////////////////   ado here down ////////////////////////////

capture program drop vx_dbel
	
program vx_dbel, rclass
	version 12
	syntax varlist  [, signif(real 0.05) repsnum(real 10000) sided(string) method(string)]
	
	preserve
  
	quietly putmata datmatx=(`varlist'), replace
	
	local state = c(seed)
	set seed `state'
	
	///////////////////////// method monte carlo ///////////////////////
	if (strmatch(strlower("`method'"),"mc") )        mata: dbelmc(datmatx,`repsnum',"`varlist'","`sided'")
						
	
	/////////////////////////// method Bayes ///////////////////////////
	//// three sample
						
	mata: ncols=cols(datmatx)
	drop _all
	getmata (ncols)=ncols, replace force
	local ncolsflag=ncols
		
	if (strmatch(strlower("`method'"),"bayes") & `ncolsflag'==3)   {
						mata: nrows=nrows(datmatx)
						mata: reg=reg(datmatx)
					
						use critval3s, clear
						quietly putmata crit3s=(*), replace omitmissing
						
						mata: ts=dbeltable3(reg, nrows, crit3s, datmatx, "`varlist'", "`sided'")
				
						mata: decision=tableexact3(reg, nrows, ncols, ts, crit3s, "`sided'")
					
						drop _all
						getmata (reg)=reg, replace force
						local regflag=reg 
				
						if (`regflag'==1) {
									drop _all
									getmata (n m p cval alpha sided reg predictn predictm predictp)=crit3s, replace force
		
									gen invn=1/n
									gen n2=n^2
									gen invm=1/m
									gen m2=m^2
									gen invp=1/p
									gen p2=p^2
									qui reg cval n m p invn invm invp n2 m2 p2 alpha
									
									quietly keep in 1
									gen sig0=e(rmse)
									keep predictn predictm predictp sig0
								
									quietly putmata tempprior=(predictn predictm predictp sig0), replace 
									
									mata: betas = st_matrix("e(b)")
									mata: prior_param=getprior3(betas, tempprior, `signif')
							
									mata: decision=dbelbayes(datmatx, prior_param, "`varlist'", "`sided'",`signif')	
								}
								
						quietly drop _all
						getmata (decision)=decision, replace force
						return scalar reject=decision
			}
			
end
	

////////////////////// three sample test //////////////////////

mata:
	real scalar function qqsim3s(real scalar xn, real scalar yn, real scalar zn)
		{
			real colvector x, y, z, xyz
			real scalar log_vnk
			
			x = sort(runiform(xn,1),1)
			y = sort(runiform(yn,1),1)
			z = sort(runiform(zn,1),1)
			xyz = sort((x\y\z),1)	
			
			log_vnk=log(elr3s(x,xyz)*elr3s(y,xyz)*elr3s(z,xyz))
			//log_vnk
			
			return(log_vnk)
		}
end


mata:
	real scalar function qqt03s(real colvector x, real colvector y, real colvector z)
		{
			real colvector xyz
			real scalar xn, yn, zn, teststat
			
			xn=rows(x)
			yn=rows(y)
			zn=rows(z)
						
			x=sort(x,1)
			y=sort(y,1)
			z=sort(z,1)
			xyz=sort((x\y\z),1)
			
			teststat=log(elr3s(x,xyz)*elr3s(y,xyz)*elr3s(z,xyz))
			//teststat
			
			return(teststat)
		}
end


mata:
	real scalar function elr3s(real colvector q1, real colvector qq)
		{
			real colvector m, rm, mm, L, LL, U, UU, qL, qU, F
			real rowvector temp, prodFmat
			real matrix Fmat
			real scalar q1n, qqn, elr
			
			q1n=rows(q1)
			qqn=rows(qq)

			m=(round(q1n^0.6)::min(( round(q1n^.9),round(q1n/2) )))

			rm=m#J(q1n,1,1)
			mm=J(rows(m),1,1)#(1::q1n)

			L=mm-rm
			LL=(L:*(L:>0))+(L:<=0)
			U=mm+rm
			UU=(U:*(U:<=q1n))+(U:>q1n)*q1n

			qL=q1[LL]
			qU=q1[UU]
			
			F=mm_relrank(qq,rows(qq),qU)-mm_relrank(qq,rows(qq),qL)
			F=editvalue(F,0,1/(qqn))

			F=(2*rm):/(q1n*F)
						
			Fmat=colshape(F,q1n)
			_transpose(Fmat)
				
			prodFmat=exp(colsum(log(Fmat)))
			
			elr=min(prodFmat)
			
			return(elr)
		}
end




////////////////////////// begin and branch by test ////////////////////////


////////////// table /////////////////
mata:
real scalar reg( real matrix x)
	{
		nrows=J(1,3,0)
		reg=J(1,1,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		if (nrows[1,1]>30 | nrows[1,2]>30 | nrows[1,3]>30 )  reg=1
		
		return(reg)
	}		
end


mata:
real rowvector nrows( real matrix x)
	{
		nrows=J(1,3,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		nrows=nrows[1,1],nrows[1,2],nrows[1,3]
	
		return(nrows)
	}		
end


mata:
real scalar dbeltable(real scalar reg, real rowvector nrows, real matrix crit1s, real matrix x, string dsnlist, string testside)
	{
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
				
		if (strmatch(testside,"greater") & cols(x)==1)
			{
				crit1s=select(crit1s[.,1::13],crit1s[.,1]:==1)
				yy=-xx
				t0=wwt01s(xx,yy)
				if (reg==1) {
								crit1s=select(crit1s[.,1::13],crit1s[.,3]:>=30)
								if (30<nrows[1,1]  & nrows[1,1]<35) 	crit1s=select(crit1s[.,1::4],crit1s[.,5]:==1)
								if (35<=nrows[1,1] & nrows[1,1]<40)		crit1s=select(crit1s[.,1::4],crit1s[.,6]:==1)
								if (40<=nrows[1,1] & nrows[1,1]<50) 	crit1s=select(crit1s[.,1::4],crit1s[.,7]:==1)
								if (50<=nrows[1,1] & nrows[1,1]<60) 	crit1s=select(crit1s[.,1::4],crit1s[.,8]:==1)
								if (60<=nrows[1,1] & nrows[1,1]<80) 	crit1s=select(crit1s[.,1::4],crit1s[.,9]:==1)
								if (80<=nrows[1,1] & nrows[1,1]<100) 	crit1s=select(crit1s[.,1::4],crit1s[.,10]:==1)
								if (100<=nrows[1,1] & nrows[1,1]<150) 	crit1s=select(crit1s[.,1::4],crit1s[.,11]:==1)
								if (150<=nrows[1,1] & nrows[1,1]<200) 	crit1s=select(crit1s[.,1::4],crit1s[.,12]:==1)
								if (200<=nrows[1,1]) 					crit1s=select(crit1s[.,1::4],crit1s[.,13]:==1)		
							
								addinfo=J(rows(crit1s),2,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								crit1s=(crit1s,addinfo)
								
							}
				if (reg==0) crit1s=select(crit1s[.,1::4],crit1s[.,3]:==nrows[1,1])
				
			}
			
		if (strmatch(testside,"one.sample"))
			{
				crit1s=select(crit1s[.,1::13],crit1s[.,1]:==2)
				yy=-xx
				t0=wwt0(xx,yy)
				if (reg==1) {
								crit1s=select(crit1s[.,1::13],crit1s[.,3]:>=30)
								if (30<nrows[1,1]  & nrows[1,1]<35) 	crit1s=select(crit1s[.,1::4],crit1s[.,5]:==1)
								if (35<=nrows[1,1] & nrows[1,1]<40)		crit1s=select(crit1s[.,1::4],crit1s[.,6]:==1)
								if (40<=nrows[1,1] & nrows[1,1]<50) 	crit1s=select(crit1s[.,1::4],crit1s[.,7]:==1)
								if (50<=nrows[1,1] & nrows[1,1]<60) 	crit1s=select(crit1s[.,1::4],crit1s[.,8]:==1)
								if (60<=nrows[1,1] & nrows[1,1]<80) 	crit1s=select(crit1s[.,1::4],crit1s[.,9]:==1)
								if (80<=nrows[1,1] & nrows[1,1]<100) 	crit1s=select(crit1s[.,1::4],crit1s[.,10]:==1)
								if (100<=nrows[1,1] & nrows[1,1]<150) 	crit1s=select(crit1s[.,1::4],crit1s[.,11]:==1)
								if (150<=nrows[1,1] & nrows[1,1]<200) 	crit1s=select(crit1s[.,1::4],crit1s[.,12]:==1)
								if (200<=nrows[1,1]) 					crit1s=select(crit1s[.,1::4],crit1s[.,13]:==1)		
								
								addinfo=J(rows(crit1s),2,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								crit1s=(crit1s,addinfo)
								
							}
				if (reg==0) crit1s=select(crit1s[.,1::4],crit1s[.,3]:==nrows[1,1])			
	
			}
	
		return(t0)
	}
		
end


mata:
real scalar dbeltable2(real scalar reg, real rowvector nrows, real matrix crit2s, real matrix x, string dsnlist, string testside)
	{		
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
				
		//////////////////////////////////////////// two sample /////////////////////////////////////////////

		if (strmatch(testside,"greater") & cols(x)==2)
			{
				crit2s=select(crit2s[.,1::14],crit2s[.,5]:==1) //selects sided		
				t0=qqt01s(xx,yy)
				
				if (reg==1) {
								crit2s=select(crit2s[.,1::14],crit2s[.,1]:>30 :| crit2s[.,2]:>30)
								
								if ((30<nrows[1,1]  & nrows[1,1]<35)   |  (30<nrows[1,2]  & nrows[1,2]<35))		crit2s=select(crit2s[.,1::14],crit2s[.,6]:==1)
								if ((35<=nrows[1,1] & nrows[1,1]<40)   |  (35<=nrows[1,2] & nrows[1,2]<40))		crit2s=select(crit2s[.,1::14],crit2s[.,7]:==1)
								if ((40<=nrows[1,1] & nrows[1,1]<50)   |  (40<nrows[1,2]  & nrows[1,2]<50))		crit2s=select(crit2s[.,1::14],crit2s[.,8]:==1)
								if ((50<=nrows[1,1] & nrows[1,1]<60)   |  (50<nrows[1,2]  & nrows[1,2]<60))		crit2s=select(crit2s[.,1::14],crit2s[.,9]:==1)
								if ((60<=nrows[1,1] & nrows[1,1]<80)   |  (60<nrows[1,2]  & nrows[1,2]<80))		crit2s=select(crit2s[.,1::14],crit2s[.,10]:==1)
								if ((80<=nrows[1,1] & nrows[1,1]<100)  |  (80<nrows[1,2]  & nrows[1,2]<100))	crit2s=select(crit2s[.,1::14],crit2s[.,11]:==1)
								if ((100<=nrows[1,1] & nrows[1,1]<150) |  (100<nrows[1,2]  & nrows[1,2]<150))	crit2s=select(crit2s[.,1::14],crit2s[.,12]:==1)
								if ((150<=nrows[1,1] & nrows[1,1]<200) |  (1500<nrows[1,2] & nrows[1,2]<200))	crit2s=select(crit2s[.,1::14],crit2s[.,13]:==1)
								if (200<=nrows[1,1] | 200<=nrows[1,2]) 											crit2s=select(crit2s[.,1::14],crit2s[.,14]:==1)		
								
								if (30>=nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:==nrows[1,1])
								if (30>=nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:==nrows[1,2])
								if (30<nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:>=30)
								if (30<nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:>=30)
								
								crit2s=crit2s[.,1::5]
								
								addinfo=J(rows(crit2s),3,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								addinfo[.,3]=addinfo[.,1]:*nrows[1,2]
								crit2s=(crit2s,addinfo)	
							}	
		
				if (reg==0) 	crit2s=select(crit2s[.,1::5],crit2s[.,1]:==nrows[1,1] :& crit2s[.,2]:==nrows[1,2])
									
			}

	
		if (strmatch(testside,"two.sample") & cols(x)==2)
			{
				crit2s=select(crit2s[.,1::14],crit2s[.,5]:==2)
				t0=qqt0(xx,yy)

				if (reg==1) {
								crit2s=select(crit2s[.,1::14],crit2s[.,1]:>30 :| crit2s[.,2]:>30)
								
								if ((30<nrows[1,1]  & nrows[1,1]<35)   |  (30<nrows[1,2]  & nrows[1,2]<35))		crit2s=select(crit2s[.,1::14],crit2s[.,6]:==1)
								if ((35<=nrows[1,1] & nrows[1,1]<40)   |  (35<=nrows[1,2] & nrows[1,2]<40))		crit2s=select(crit2s[.,1::14],crit2s[.,7]:==1)
								if ((40<=nrows[1,1] & nrows[1,1]<50)   |  (40<nrows[1,2]  & nrows[1,2]<50))		crit2s=select(crit2s[.,1::14],crit2s[.,8]:==1)
								if ((50<=nrows[1,1] & nrows[1,1]<60)   |  (50<nrows[1,2]  & nrows[1,2]<60))		crit2s=select(crit2s[.,1::14],crit2s[.,9]:==1)
								if ((60<=nrows[1,1] & nrows[1,1]<80)   |  (60<nrows[1,2]  & nrows[1,2]<80))		crit2s=select(crit2s[.,1::14],crit2s[.,10]:==1)
								if ((80<=nrows[1,1] & nrows[1,1]<100)  |  (80<nrows[1,2]  & nrows[1,2]<100))	crit2s=select(crit2s[.,1::14],crit2s[.,11]:==1)
								if ((100<=nrows[1,1] & nrows[1,1]<150) |  (100<nrows[1,2]  & nrows[1,2]<150))	crit2s=select(crit2s[.,1::14],crit2s[.,12]:==1)
								if ((150<=nrows[1,1] & nrows[1,1]<200) |  (1500<nrows[1,2] & nrows[1,2]<200))	crit2s=select(crit2s[.,1::14],crit2s[.,13]:==1)
								if (200<=nrows[1,1] | 200<=nrows[1,2]) 											crit2s=select(crit2s[.,1::14],crit2s[.,14]:==1)		
								
								if (30>=nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:==nrows[1,1])
								if (30>=nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:==nrows[1,2])
								if (30<nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:>=30)
								if (30<nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:>=30)
								
								crit2s=crit2s[.,1::5]
															
								addinfo=J(rows(crit2s),3,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								addinfo[.,3]=addinfo[.,1]:*nrows[1,2]
								crit2s=(crit2s,addinfo)	
							}
							
				if (reg==0) 	crit2s=select(crit2s[.,1::5],crit2s[.,1]:==nrows[1,1] :& crit2s[.,2]:==nrows[1,2])
		
			}
			
		return(t0)
	}
		
end


mata:

real scalar tableexact(real scalar reg,real rowvector nrows, real scalar ncols, real scalar ts, real matrix crit1s, string testside)
	{
		real scalar decision
		decision=.
		
		//////// one sample -- 1s & 2s  ////////
		if (ncols==1 & reg==0)
			{
				Tcrit1s=crit1s'		
				if (ts>Tcrit1s[4,5]) 	decision=1
				if (Tcrit1s[4,5]>=ts) 	decision=0				
			}
		return(decision)
	}
		
end


mata:

real scalar tableexact2(real scalar reg,real rowvector nrows, real scalar ncols, real scalar ts, real matrix crit2s, string testside)
	{
		real scalar decision
		decision=.
		
		//////// two sample -- 1s & 2s  ////////
		if (ncols==2 & reg==0)
			{
				Tcrit2s=crit2s'	
				if (ts>Tcrit2s[3,5]) 	decision=1
				if (Tcrit2s[3,5]>=ts) 	decision=0				
			}
		return(decision)
	}
		
end


mata:
real scalar dbeltable3(real scalar reg, real rowvector nrows, real matrix crit3s, real matrix x, string dsnlist, string testside)
	{		
		real scalar t0
		
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
			
		//////////////////////////////////////////// two sample /////////////////////////////////////////////

		if (strmatch(testside,"three.sample") & cols(x)==3)
			{
				t0=qqt03s(xx,yy,zz)
	
				if (reg==1) {
								crit3s=select(crit3s[.,1::15],crit3s[.,1]:>30 :| crit3s[.,2]:>30 :|crit3s[.,3]:>30)
							
								if ((30<nrows[1,1]  & nrows[1,1]<35)   |  (30<nrows[1,2]  & nrows[1,2]<35)	  	|  (30<nrows[1,3]  & nrows[1,3]<35))		crit3s=select(crit3s[.,1::15],crit3s[.,7]:==1)
								if ((35<=nrows[1,1] & nrows[1,1]<40)   |  (35<=nrows[1,2] & nrows[1,2]<40)		|  (35<nrows[1,3]  & nrows[1,3]<40))		crit3s=select(crit3s[.,1::15],crit3s[.,8]:==1)
								if ((40<=nrows[1,1] & nrows[1,1]<50)   |  (40<nrows[1,2]  & nrows[1,2]<50)		|  (40<nrows[1,3]  & nrows[1,3]<50))		crit3s=select(crit3s[.,1::15],crit3s[.,9]:==1)
								if ((50<=nrows[1,1] & nrows[1,1]<60)   |  (50<nrows[1,2]  & nrows[1,2]<60)		|  (50<nrows[1,3]  & nrows[1,3]<60))		crit3s=select(crit3s[.,1::15],crit3s[.,10]:==1)
								if ((60<=nrows[1,1] & nrows[1,1]<80)   |  (60<nrows[1,2]  & nrows[1,2]<80)		|  (60<nrows[1,3]  & nrows[1,3]<80))		crit3s=select(crit3s[.,1::15],crit3s[.,11]:==1)	
								if ((80<=nrows[1,1] & nrows[1,1]<100)  |  (80<nrows[1,2]  & nrows[1,2]<100)		|  (80<nrows[1,3]  & nrows[1,3]<100))		crit3s=select(crit3s[.,1::15],crit3s[.,12]:==1)
								if ((100<=nrows[1,1] & nrows[1,1]<150) |  (100<nrows[1,2] & nrows[1,2]<150)		|  (100<nrows[1,3] & nrows[1,3]<150))		crit3s=select(crit3s[.,1::15],crit3s[.,13]:==1)
								if ((150<=nrows[1,1] & nrows[1,1]<200) |  (150<nrows[1,2] & nrows[1,2]<200)		|  (150<nrows[1,3] & nrows[1,3]<200))		crit3s=select(crit3s[.,1::15],crit3s[.,14]:==1)
								if (200<=nrows[1,1] | 200<=nrows[1,2] |  200<=nrows[1,3])																	crit3s=select(crit3s[.,1::15],crit3s[.,15]:==1)		
							
								if (30>=nrows[1,1])	crit3s=select(crit3s[.,1::15],crit3s[.,1]:==nrows[1,1])
								if (30>=nrows[1,2])	crit3s=select(crit3s[.,1::15],crit3s[.,2]:==nrows[1,2])
								if (30>=nrows[1,3])	crit3s=select(crit3s[.,1::15],crit3s[.,3]:==nrows[1,3])
								if (30<nrows[1,1])	crit3s=select(crit3s[.,1::15],crit3s[.,1]:>30)
								if (30<nrows[1,2])	crit3s=select(crit3s[.,1::15],crit3s[.,2]:>30)
								if (30<nrows[1,3])	crit3s=select(crit3s[.,1::15],crit3s[.,3]:>30)
								
								crit3s=crit3s[.,1::6]
								
								addinfo=J(rows(crit3s),4,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								addinfo[.,3]=addinfo[.,1]:*nrows[1,2]
								addinfo[.,4]=addinfo[.,1]:*nrows[1,3]
								crit3s=(crit3s,addinfo)	
							
							}	
		
				if (reg==0) 	crit3s=select(crit3s[.,1::6],crit3s[.,1]:==nrows[1,1] :& crit3s[.,2]:==nrows[1,2] :& crit3s[.,3]:==nrows[1,3])
								
			}
		return(t0)
	}
		
end


mata:

real scalar tableexact3(real scalar reg,real rowvector nrows, real scalar ncols, real scalar ts, real matrix crit3s, string testside)
	{
		string test, pvalue
		real scalar decision
		
		//////// two sample -- 1s & 2s  ////////
		if (ncols==3 & reg==0)
			{
				Tcrit3s=crit3s'	
				//Tcrit3s
				//ts
				if (ts>Tcrit3s[4,5]) 	decision=1
				if (Tcrit3s[4,5]>=ts) 	decision=0				
			}
		return(decision)	
	}
		
end


mata: 
	function invreg(x,c1,c2,c3) return((c1+c2*x+c3*x^0.5)^2)
end

	
		

////////////// bayes /////////////////


mata: 
	real matrix function getprior3(real matrix betas, real matrix tempprior, real scalar alpha)
		{
			n=tempprior[1,1]
			m=tempprior[1,2]
			p=tempprior[1,3]
			mu0=betas[1,11] + betas[1,1]*n + betas[1,2]*m + betas[1,3]*p + betas[1,4]*(1/n) + ///
								betas[1,5]*(1/m) + betas[1,6]*(1/p) + 	betas[1,7]*(n^2) + ///
								betas[1,8]*(m^2) + betas[1,9]*(p^2) + betas[1,10]*alpha 
			priorparam=(mu0,tempprior[1,4])
			
			return(priorparam)	
		}
end


mata: 
	real matrix function getTk3(real colvector xx, real colvector yy, real colvector zz, real scalar tk, string testside)
		{
			mc=J(tk,1,0)
			nn=1
			
			//////// three sample ////////
			if (strmatch(testside,"three.sample"))
				{
					while(nn<=tk) {   
						  mc[nn,1]=qqsim3s(rows(xx),rows(yy),rows(zz))
						  ++nn
						}
					mc=sort(mc,1)
				}
				
			
			return(mc)	
		}
end


mata: 
	real matrix function getJk(real scalar tk, real scalar alpha)
		{
			L1=(1-alpha)*tk - (tk^0.5)*ln(tk)
			LL=(1,round(L1))
			U1=(1-alpha)*tk + (tk^0.5)*ln(tk)
			UU=(round(U1),tk)
			Jk=(rowmax(LL), rowmin(UU))
			
			return(Jk)
		}
end


mata: 
	real scalar function getqhatca(real colvector TT, real rowvector JJ, real colvector Jk, real scalar tk, string testside, real scalar u0, real scalar sig0, real scalar alpha)
		{
			Tk=TT[JJ[1,1]::JJ[1,2],1]
			Tkminus=(.\Tk[1::rows(Tk)-1,1])
			//Tk,Tkminus
			
			exp_part=exp( (-tk/(2*alpha*(1-alpha))):*(((1-alpha):-(Jk:/tk)):^2) )
			integral_pi=normal((Tk:-u0):/sig0) - normal((Tkminus:-u0):/sig0) 					
			integral_qpi=((sig0/(2*pi()))^0.5):*( exp(-((Tkminus:-u0):^2):/(2*sig0)):-exp(-((Tk:-u0):^2):/(2*sig0))):+(u0:*integral_pi)
			qhatca=colsum(exp_part:*integral_qpi)/colsum(exp_part:*integral_pi)
			
			return(qhatca)
		}
end


mata: 
	real colvector function getf2t2qhat1a3(real colvector xx, real colvector yy,  real colvector zz, string testside, real scalar qhat1a)
		{
			t2=200
			T2=getTk3(xx, yy, zz, t2, testside)
			sig2=variance(T2)
			h=1.06*(sig2^0.5)*(t2^-0.2)
			f2t2qhat1a=(1/t2)*sqrt(1/(2*pi()*(h^2)))*colsum( exp( (-1/(2*(h^2))):*((qhat1a:-T2):^2)) )
			
			f2t2qhat1aplus=f2t2qhat1a\T2
			return(f2t2qhat1aplus)
		}
end


mata: 
	real scalar function dbelbayes(real matrix x, real rowvector prior_param, string dsnlist, string testside, real scalar alpha)
		{
			if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
			if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
			if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
			
			//if (cols(x)==1) dbelbayes1s(xx, prior_param, testside, alpha)
			//if (cols(x)==2) decision=dbelbayes2s(xx, yy,  prior_param, testside, alpha)
			if (cols(x)==3) decision=dbelbayes3s(xx, yy, zz, prior_param, testside, alpha)
			//decision
			return(decision)
		}
end


mata: 
	real scalar function dbelbayes3s(real colvector xx, real colvector yy, real colvector zz, real rowvector prior_param, string testside, real scalar alpha)
		{
			real scalar t0, t1, t2, tt, sig0, u0, V1, qhat1a, f2t2qhat1a, qhatca, decision
			real rowvector JJ
			real colvector T2, J1, TT, f2t2qhat1aplus
			string test
			
			sig0=prior_param[1,2]^2
			u0=prior_param[1,1]
						
			t1=200
			JJ=getJk(t1,alpha)
			J1=(JJ[1,1]::JJ[1,2])
			TT=getTk3(xx, yy, zz, t1, testside)
				
			t2=200
			
			while(t1<=35000) {   
						//first cumulative sample
						qhat1a=getqhatca(TT, JJ, J1, t1, testside, u0, sig0, alpha)
						
						//second sample t2=200
						f2t2qhat1aplus=getf2t2qhat1a3(xx, yy, zz, testside, qhat1a)
						f2t2qhat1a=f2t2qhat1aplus[1,1]
						T2=f2t2qhat1aplus[2::rows(f2t2qhat1aplus),1]
								
						// get variance estimator
						V1=(mm_relrank(T2,rows(T2),qhat1a)*(1-mm_relrank(T2,rows(T2),qhat1a))/((f2t2qhat1a^2)*t2))
													
						// increment T1
						t1=t1+t2
						tt=t1
						TT=sort(TT\T2,1)
						JJ=getJk(t1,alpha)
						J1=(JJ[1,1]::JJ[1,2])
						
						if (V1<=sig0) {
								//stop procedure
								t1=999999
							}
						//tt
					}
					
			qhatca=getqhatca(TT, JJ, J1, tt, testside, u0, sig0, alpha)
			
			if (strmatch(testside,"three.sample"))  {
						t0=qqt03s(xx,yy,zz)
						test="DBEL (3-sample)"
					}
				
			if (t0>qhatca) 	decision=1
			if (t0<=qhatca) decision=0
			//decision
			
			return(decision)
		}
end	



/////////////////////////////////// end ado ///////////////////////////////////////

	
//vx_dbel dsn1, signif(0.05) sided("greater") method("bayes")

//vx_dbel dsn1 , signif(0.05) sided("one.sample") method("bayes")

	
	//generate test data to analyze
	//mata: dsn1 = runiform(16,1):-0.5
	//mata: dsn1 = rgamma(34, 1, 9, 0.5):-7
	//mata: dsn1 = rnormal(91, 1, 0, 1):+0.2
	//mata: dsn1 = rgamma(34, 1, 9, 0.5):-7


/////////////////////////// A /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(15, 1, 0, 1)
	mata: dsn3 = (-1:+(2:*runiform(20,1)))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(25, 1, 0, 1)
	mata: dsn3 = (-1:+(2:*runiform(25,1)))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(37, 1, 0, 1)
	mata: dsn2 = rnormal(45, 1, 0, 1)
	mata: dsn3 = (-1:+(2:*runiform(50,1)))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3	
	
mata: mcA=powerw1\powerw2\powerw3
mata: mcA=(J(3,1,1),mcA)
mata: mcA	

/////////////////////////// B /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25, 1, 1, 1)
	mata: dsn2 = rgamma(15, 1, 1, 1)
	mata: dsn3 = exp(rnormal(20, 1, 0, 1))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	

	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = rgamma(25, 1, 1, 1)
	mata: dsn2 = rgamma(25, 1, 1, 1)
	mata: dsn3 = exp(rnormal(25, 1, 0, 1))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(37, 1, 1, 1)
	mata: dsn2 = rgamma(45, 1, 1, 1)
	mata: dsn3 = exp(rnormal(50, 1, 0, 1))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3		

mata: mcB=powerw1\powerw2\powerw3
mata: mcB=(J(3,1,1),mcB)
mata: mcB		
		
	

/////////////////////////// E /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(15, 1, 0, 1)
	mata: dsn3 = rnormal(20, 1, 0, 2.25)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(25, 1, 0, 1)
	mata: dsn3 = rnormal(25, 1, 0, 2.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(37, 1, 0, 1)
	mata: dsn2 = rnormal(45, 1, 0, 1)
	mata: dsn3 = rnormal(50, 1, 0, 2.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3		

mata: mcE=powerw1\powerw2\powerw3
mata: mcE=(J(3,1,1),mcE)
mata: mcE		
		
	

/////////////////////////// F /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(15, 1, 0, 1)
	mata: dsn3 = rnormal(20, 1, 0, 0.25)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(25, 1, 0, 1)
	mata: dsn3 = rnormal(25, 1, 0, 0.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(37, 1, 0, 1)
	mata: dsn2 = rnormal(45, 1, 0, 1)
	mata: dsn3 = rnormal(50, 1, 0, 0.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3		

mata: mcF=powerw1\powerw2\powerw3
mata: mcF=(J(3,1,1),mcF)
mata: mcF			
		
	

/////////////////////////// J /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25, 1, 1, 1) 
	mata: dsn2 =  exp(rnormal(15, 1, 0, 1))
	mata: dsn3 =  runiform(20, 1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25, 1, 1, 1) 
	mata: dsn2 =  exp(rnormal(25, 1, 0, 1))
	mata: dsn3 =  runiform(25, 1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(37, 1, 1, 1) 
	mata: dsn2 = exp(rnormal(45, 1, 0, 1))
	mata: dsn3 = runiform(50, 1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcJ=powerw1\powerw2\powerw3
mata: mcJ=(J(3,1,1),mcJ)
mata: mcJ			
	

/////////////////////////// M /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = runiform(25,1)
	mata: dsn2 = runiform(15,1)
	mata: dsn3 = runiform(20,1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = runiform(25,1)
	mata: dsn2 = runiform(25,1)
	mata: dsn3 = runiform(25,1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = runiform(37,1)
	mata: dsn2 = runiform(45,1)
	mata: dsn3 = runiform(50,1)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcM=powerw1\powerw2\powerw3
mata: mcM=(J(3,1,1),mcM)
mata: mcM				
		


/////////////////////////// N /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25,1,0,1)
	mata: dsn2 = rnormal(15,1,0,1)
	mata: dsn3 = rnormal(20,1,0,1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25,1,0,1)
	mata: dsn2 = rnormal(25,1,0,1)
	mata: dsn3 = rnormal(25,1,0,1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = rnormal(37,1,0,1)
	mata: dsn2 = rnormal(45,1,0,1)
	mata: dsn3 = rnormal(50,1,0,1)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcN=powerw1\powerw2\powerw3
mata: mcN=(J(3,1,1),mcN)
mata: mcN




/////////////////////////// P /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25,1,1,1)
	mata: dsn2 = rgamma(15,1,1,1)
	mata: dsn3 = rgamma(20,1,1,1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25,1,1,1)
	mata: dsn2 = rgamma(25,1,1,1)
	mata: dsn3 = rgamma(25,1,1,1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = rgamma(37,1,1,1)
	mata: dsn2 = rgamma(45,1,1,1)
	mata: dsn3 = rgamma(50,1,1,1)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("bayes")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcP=powerw1\powerw2\powerw3
mata: mcP=(J(3,1,1),mcP)
mata: mcP




mata: mc=(mcA\mcB\mcE\mcF\mcJ\mcM\mcN\mcP)
		
	
cd "E:\STATA Projects\Vexler2- DBEL Project-Tanajian-for WEB SITE\proportion rejected programs\threesample"
getmata (design n m p c05 power)=mc, replace force
export excel design n m p c05 power using prop_rej_3sbayes_sub, firstrow(variables) replace
			
