version 12
clear all

cd "C:\ado\personal"	


capture program drop vx_dbel
	

program vx_dbel, rclass
	version 12
	syntax varlist  [, repsnum(real 10000) sided(string) method(string)]
	
	capture mata: mata drop vx_dbel.invreg()
	preserve
  
	quietly putmata datmatx=(`varlist'), replace
	
	local state = c(seed)
	set seed `state'
	
	///////////////////////// method monte carlo ///////////////////////
	if (strmatch(strlower("`method'"),"mc") )        mata: dbelmc(datmatx,`repsnum',"`varlist'","`sided'")
	
	/////////////////////////// method table ///////////////////////////
	//// three sample
	
	mata: ncols=cols(datmatx)
	drop _all
	getmata (ncols)=ncols, replace force
	local ncolsflag=ncols
	
	if (strmatch(strlower("`method'"),"reg") & `ncolsflag'==3 )   	{	
						mata: ncols=cols(datmatx)
						mata: nrows=nrows(datmatx)
						mata: reg=reg(datmatx)
						
						use critval3s, clear
						quietly putmata crit3s=(*), replace omitmissing
						
						mata: ts=dbeltable3(reg, nrows, crit3s, datmatx, "`varlist'", "`sided'")
					
						mata: decision=tableexact3(reg, nrows, ncols, ts, crit3s, "`sided'")
					
						drop _all
						getmata (reg)=reg, replace force
						local regflag=reg 
					
						if (`regflag'==1) {
									drop _all
									getmata (n m p cval alpha sided reg predictn predictm predictp)=crit3s, replace force
		
									gen invn=1/n
									gen n2=n^2
									gen invm=1/m
									gen m2=m^2
									gen invp=1/p
									gen p2=p^2
									gen alphahalf=alpha^0.5
									qui reg cval n m p invn invm invp n2 m2 p2 alpha alphahalf
									
									mata: betas = st_matrix("e(b)")
									mata: decision=getroot3(crit3s, betas, ts, "`sided'")
								}	
							
						quietly drop _all
						getmata (decision)=decision, replace force
						return scalar reject=decision		
			}		
end



mata:
	void function getoptim(todo,b,c1,c2,c3,crit,g,H)
	{
		crit=(c1+c2*b+c3*b^0.5)^2
	}
end


mata: 
	function invreg(x,c1,c2,c3) return(c1+c2*x+c3*x^0.5)
end

mata: 
	real scalar function getroot3(real matrix crit3s, real rowvector betas, real scalar ts, string sided)
		{ 
			real scalar c1, c2, c3, testroot, rc, pvalue, max_cval, min_cval
			
			pvalue=9.9999 
			
			npred=crit3s[1,8]
			mpred=crit3s[1,9]
			ppred=crit3s[1,10]
			max_cval=colmax(crit3s[.,4])
			min_cval=colmin(crit3s[.,4])
			
			c1=betas[1,12] + betas[1,1]*npred + betas[1,2]*mpred + betas[1,3]*ppred + betas[1,4]*(1/npred) + ///
								betas[1,5]*(1/mpred) + betas[1,6]*(1/ppred) + 	betas[1,7]*(npred^2) + ///
								betas[1,8]*(mpred^2) + betas[1,9]*(ppred^2) - ts
			c2=betas[1,10]
			c3=betas[1,11]
					
			rc=mm_root(pvalue=.,&invreg(),0,1,0,10000,c1,c2,c3)
	
			pvalue=round(pvalue,10e-4)
			
			if (ts>max_cval)  pvalue=0
			if (ts<min_cval)  pvalue=1
			
			if (pvalue<0.05) 	decision=1
			if (pvalue>=0.05)   decision=0
						
			return(decision)
			
		}
end



////////////////////// three sample test //////////////////////

mata:
	real scalar function qqsim3s(real scalar xn, real scalar yn, real scalar zn)
		{
			real colvector x, y, z, xyz
			real scalar log_vnk
			
			x = sort(runiform(xn,1),1)
			y = sort(runiform(yn,1),1)
			z = sort(runiform(zn,1),1)
			xyz = sort((x\y\z),1)	
			
			log_vnk=log(elr3s(x,xyz)*elr3s(y,xyz)*elr3s(z,xyz))
			//log_vnk
			
			return(log_vnk)
		}
end


mata:
	real scalar function qqt03s(real colvector x, real colvector y, real colvector z)
		{
			real colvector xyz
			real scalar xn, yn, zn, teststat
			
			xn=rows(x)
			yn=rows(y)
			zn=rows(z)
						
			x=sort(x,1)
			y=sort(y,1)
			z=sort(z,1)
			xyz=sort((x\y\z),1)
			
			teststat=log(elr3s(x,xyz)*elr3s(y,xyz)*elr3s(z,xyz))
			//teststat
			
			return(teststat)
		}
end


mata:
	real scalar function elr3s(real colvector q1, real colvector qq)
		{
			real colvector m, rm, mm, L, LL, U, UU, qL, qU, F
			real rowvector temp, prodFmat
			real matrix Fmat
			real scalar q1n, qqn, elr
			
			q1n=rows(q1)
			qqn=rows(qq)

			m=(round(q1n^0.6)::min(( round(q1n^.9),round(q1n/2) )))

			rm=m#J(q1n,1,1)
			mm=J(rows(m),1,1)#(1::q1n)

			L=mm-rm
			LL=(L:*(L:>0))+(L:<=0)
			U=mm+rm
			UU=(U:*(U:<=q1n))+(U:>q1n)*q1n

			qL=q1[LL]
			qU=q1[UU]
			
			F=mm_relrank(qq,rows(qq),qU)-mm_relrank(qq,rows(qq),qL)
			F=editvalue(F,0,1/(qqn))

			F=(2*rm):/(q1n*F)
						
			Fmat=colshape(F,q1n)
			_transpose(Fmat)
				
			prodFmat=exp(colsum(log(Fmat)))
			
			elr=min(prodFmat)
			
			return(elr)
		}
end




////////////////////////// begin and branch by test ////////////////////////

//////////////// mc /////////////////
mata:
	void dbelmc(real matrix x, real scalar rnum, string dsnlist, string testside)
	{
		
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
		
		mc=J(rnum,1,0)
			
		///////// three sample /////////
		
		if (strmatch(testside,"three.sample"))
			{
				nn=1
				while(nn<=rnum) {   
					  mc[nn,1]=qqsim3s(rows(xx),rows(yy),rows(zz))
					  ++nn
					}
				mc=sort(mc,1)
				t0=qqt03s(xx,yy,zz)
				pvalue=(colsum(mc:>=t0))/rnum
				
				test="DBEL (3-sample)"
				
				printf("{txt}{space 16}{c |}  T0               P-value\n")
				printf("{hline 16}{c +}{hline 24}\n")
				printf("{txt}%12s {c |} {res}%10.0g %10.0g\n",test, t0, pvalue)
			}
		}
		
end

////////////// table /////////////////
mata:
real scalar reg( real matrix x)
	{
		nrows=J(1,3,0)
		reg=J(1,1,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		if (nrows[1,1]>30 | nrows[1,2]>30 | nrows[1,3]>30 )  reg=1
		
		return(reg)
	}		
end

mata:
real rowvector nrows( real matrix x)
	{
		nrows=J(1,3,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		nrows=nrows[1,1],nrows[1,2],nrows[1,3]
	
		return(nrows)
	}		
end


mata:
real scalar dbeltable3(real scalar reg, real rowvector nrows, real matrix crit3s, real matrix x, string dsnlist, string testside)
	{		
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
				
		//////////////////////////////////////////// two sample /////////////////////////////////////////////

		if (strmatch(testside,"three.sample") & cols(x)==3)
			{
				t0=qqt03s(xx,yy,zz)
			
				if (reg==1) {
								crit3s=select(crit3s[.,1::15],crit3s[.,1]:>30 :| crit3s[.,2]:>30 :|crit3s[.,3]:>30)
							
								if ((30<nrows[1,1]  & nrows[1,1]<35)   |  (30<nrows[1,2]  & nrows[1,2]<35)	  	|  (30<nrows[1,3]  & nrows[1,3]<35))		crit3s=select(crit3s[.,1::15],crit3s[.,7]:==1)
								if ((35<=nrows[1,1] & nrows[1,1]<40)   |  (35<=nrows[1,2] & nrows[1,2]<40)		|  (35<nrows[1,3]  & nrows[1,3]<40))		crit3s=select(crit3s[.,1::15],crit3s[.,8]:==1)
								if ((40<=nrows[1,1] & nrows[1,1]<50)   |  (40<nrows[1,2]  & nrows[1,2]<50)		|  (40<nrows[1,3]  & nrows[1,3]<50))		crit3s=select(crit3s[.,1::15],crit3s[.,9]:==1)
								if ((50<=nrows[1,1] & nrows[1,1]<60)   |  (50<nrows[1,2]  & nrows[1,2]<60)		|  (50<nrows[1,3]  & nrows[1,3]<60))		crit3s=select(crit3s[.,1::15],crit3s[.,10]:==1)
								if ((60<=nrows[1,1] & nrows[1,1]<80)   |  (60<nrows[1,2]  & nrows[1,2]<80)		|  (60<nrows[1,3]  & nrows[1,3]<80))		crit3s=select(crit3s[.,1::15],crit3s[.,11]:==1)	
								if ((80<=nrows[1,1] & nrows[1,1]<100)  |  (80<nrows[1,2]  & nrows[1,2]<100)		|  (80<nrows[1,3]  & nrows[1,3]<100))		crit3s=select(crit3s[.,1::15],crit3s[.,12]:==1)
								if ((100<=nrows[1,1] & nrows[1,1]<150) |  (100<nrows[1,2] & nrows[1,2]<150)		|  (100<nrows[1,3] & nrows[1,3]<150))		crit3s=select(crit3s[.,1::15],crit3s[.,13]:==1)
								if ((150<=nrows[1,1] & nrows[1,1]<200) |  (150<nrows[1,2] & nrows[1,2]<200)		|  (150<nrows[1,3] & nrows[1,3]<200))		crit3s=select(crit3s[.,1::15],crit3s[.,14]:==1)
								if (200<=nrows[1,1] | 200<=nrows[1,2] |  200<=nrows[1,3])																	crit3s=select(crit3s[.,1::15],crit3s[.,15]:==1)		
							
								if (30>=nrows[1,1])	crit3s=select(crit3s[.,1::15],crit3s[.,1]:==nrows[1,1])
								if (30>=nrows[1,2])	crit3s=select(crit3s[.,1::15],crit3s[.,2]:==nrows[1,2])
								if (30>=nrows[1,3])	crit3s=select(crit3s[.,1::15],crit3s[.,3]:==nrows[1,3])
								if (30<nrows[1,1])	crit3s=select(crit3s[.,1::15],crit3s[.,1]:>30)
								if (30<nrows[1,2])	crit3s=select(crit3s[.,1::15],crit3s[.,2]:>30)
								if (30<nrows[1,3])	crit3s=select(crit3s[.,1::15],crit3s[.,3]:>30)
								
								crit3s=crit3s[.,1::6]
								
								addinfo=J(rows(crit3s),4,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								addinfo[.,3]=addinfo[.,1]:*nrows[1,2]
								addinfo[.,4]=addinfo[.,1]:*nrows[1,3]
								crit3s=(crit3s,addinfo)	
							
							}	
		
				if (reg==0) 	crit3s=select(crit3s[.,1::6],crit3s[.,1]:==nrows[1,1] :& crit3s[.,2]:==nrows[1,2] :& crit3s[.,3]:==nrows[1,3])
								
			}
		return(t0)
	}
		
end


mata:

real scalar tableexact3(real scalar reg,real rowvector nrows, real scalar ncols, real scalar ts, real matrix crit3s, string testside)
	{
		real scalar decision
		decision=.
		
		//////// three sample  ////////
		if (ncols==3 & reg==0)
			{
				Tcrit3s=crit3s'	
				if (ts>Tcrit3s[4,5]) 	decision=1
				if (Tcrit3s[4,5]>=ts) 	decision=0				
			}
		return(decision)
	}
		
end


/////////////////////////////////// end ado ///////////////////////////////////////

	
//vx_dbel dsn1, signif(0.05) sided("greater") method("reg")

//vx_dbel dsn1 , signif(0.05) sided("one.sample") method("reg")

	
	//generate test data to analyze
	//mata: dsn1 = runiform(16,1):-0.5
	//mata: dsn1 = rgamma(34, 1, 9, 0.5):-7
	//mata: dsn1 = rnormal(91, 1, 0, 1):+0.2
	//mata: dsn1 = rgamma(34, 1, 9, 0.5):-7


/////////////////////////// A /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(15, 1, 0, 1)
	mata: dsn3 = (-1:+(2:*runiform(20,1)))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(25, 1, 0, 1)
	mata: dsn3 = (-1:+(2:*runiform(25,1)))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(37, 1, 0, 1)
	mata: dsn2 = rnormal(45, 1, 0, 1)
	mata: dsn3 = (-1:+(2:*runiform(50,1)))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3	
	
mata: mcA=powerw1\powerw2\powerw3
mata: mcA=(J(3,1,1),mcA)
mata: mcA	

/////////////////////////// B /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25, 1, 1, 1)
	mata: dsn2 = rgamma(15, 1, 1, 1)
	mata: dsn3 = exp(rnormal(20, 1, 0, 1))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	

	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = rgamma(25, 1, 1, 1)
	mata: dsn2 = rgamma(25, 1, 1, 1)
	mata: dsn3 = exp(rnormal(25, 1, 0, 1))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(37, 1, 1, 1)
	mata: dsn2 = rgamma(45, 1, 1, 1)
	mata: dsn3 = exp(rnormal(50, 1, 0, 1))
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3		

mata: mcB=powerw1\powerw2\powerw3
mata: mcB=(J(3,1,1),mcB)
mata: mcB		
		
	

/////////////////////////// E /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(15, 1, 0, 1)
	mata: dsn3 = rnormal(20, 1, 0, 2.25)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(25, 1, 0, 1)
	mata: dsn3 = rnormal(25, 1, 0, 2.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(37, 1, 0, 1)
	mata: dsn2 = rnormal(45, 1, 0, 1)
	mata: dsn3 = rnormal(50, 1, 0, 2.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3		

mata: mcE=powerw1\powerw2\powerw3
mata: mcE=(J(3,1,1),mcE)
mata: mcE		
		
	

/////////////////////////// F /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(15, 1, 0, 1)
	mata: dsn3 = rnormal(20, 1, 0, 0.25)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25, 1, 0, 1)
	mata: dsn2 = rnormal(25, 1, 0, 1)
	mata: dsn3 = rnormal(25, 1, 0, 0.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(37, 1, 0, 1)
	mata: dsn2 = rnormal(45, 1, 0, 1)
	mata: dsn3 = rnormal(50, 1, 0, 0.25)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3		

mata: mcF=powerw1\powerw2\powerw3
mata: mcF=(J(3,1,1),mcF)
mata: mcF			
		
	

/////////////////////////// J /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25, 1, 1, 1) 
	mata: dsn2 =  exp(rnormal(15, 1, 0, 1))
	mata: dsn3 =  runiform(20, 1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25, 1, 1, 1) 
	mata: dsn2 =  exp(rnormal(25, 1, 0, 1))
	mata: dsn3 =  runiform(25, 1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(37, 1, 1, 1) 
	mata: dsn2 = exp(rnormal(45, 1, 0, 1))
	mata: dsn3 = runiform(50, 1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcJ=powerw1\powerw2\powerw3
mata: mcJ=(J(3,1,1),mcJ)
mata: mcJ			
	

/////////////////////////// M /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = runiform(25,1)
	mata: dsn2 = runiform(15,1)
	mata: dsn3 = runiform(20,1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = runiform(25,1)
	mata: dsn2 = runiform(25,1)
	mata: dsn3 = runiform(25,1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = runiform(37,1)
	mata: dsn2 = runiform(45,1)
	mata: dsn3 = runiform(50,1)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcM=powerw1\powerw2\powerw3
mata: mcM=(J(3,1,1),mcM)
mata: mcM				
		


/////////////////////////// N /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25,1,0,1)
	mata: dsn2 = rnormal(15,1,0,1)
	mata: dsn3 = rnormal(20,1,0,1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(25,1,0,1)
	mata: dsn2 = rnormal(25,1,0,1)
	mata: dsn3 = rnormal(25,1,0,1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = rnormal(37,1,0,1)
	mata: dsn2 = rnormal(45,1,0,1)
	mata: dsn3 = rnormal(50,1,0,1)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcN=powerw1\powerw2\powerw3
mata: mcN=(J(3,1,1),mcN)
mata: mcN



/////////////////////////// P /////////////////////////////////
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25,1,1,1)
	mata: dsn2 = rgamma(15,1,1,1)
	mata: dsn3 = rgamma(20,1,1,1)
	
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw1=colsum(result)/10000
mata: powerw1	
	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rgamma(25,1,1,1)
	mata: dsn2 = rgamma(25,1,1,1)
	mata: dsn3 = rgamma(25,1,1,1)
	
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw2=colsum(result)/10000
mata: powerw2	
	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all	
	mata: dsn1 = rgamma(37,1,1,1)
	mata: dsn2 = rgamma(45,1,1,1)
	mata: dsn3 = rgamma(50,1,1,1)
				
	getmata dsn1 dsn2 dsn3
	
	qui vx_dbel dsn1 dsn2 dsn3, sided("three.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powerw3=colsum(result)/10000
mata: powerw3			

mata: mcP=powerw1\powerw2\powerw3
mata: mcP=(J(3,1,1),mcP)
mata: mcP




mata: mc=(mcA\mcB\mcE\mcF\mcJ\mcM\mcN\mcP)
		
	
cd "E:\STATA Projects\Vexler2- DBEL Project-Tanajian-for WEB SITE\proportion rejected programs\threesample"
getmata (design n m p c05 power)=mc, replace force
export excel design n m p c05 power using prop_rej_3sreg_sub, firstrow(variables) replace
			
