version 12
clear all

cd "C:\ado\personal"	


capture program drop vx_dbel
	
program vx_dbel, rclass
	version 12
	syntax varlist  [, repsnum(real 10000) sided(string) method(string)]
	
	capture mata: mata drop vx_dbel.invreg()
	preserve
  
	quietly putmata datmatx=(`varlist'), replace
	
	local state = c(seed)
	set seed `state'
	
	///////////////////////// method monte carlo ///////////////////////
	if (strmatch(strlower("`method'"),"mc") )        mata: dbelmc(datmatx,`repsnum',"`varlist'","`sided'")
	
	/////////////////////////// method table ///////////////////////////
	mata: ncols=cols(datmatx)
	drop _all
	getmata (ncols)=ncols, replace force
	local ncolsflag=ncols
	
	//// two sample
	if (strmatch(strlower("`method'"),"reg") & `ncolsflag'==2 )   	{	
						mata: ncols=cols(datmatx)
						mata: nrows=nrows(datmatx)
						mata: reg=reg(datmatx)
						
						use critval2s, clear
						quietly putmata crit2s=(*), replace omitmissing
						mata: ts=dbeltable2(reg, nrows, crit2s, datmatx, "`varlist'", "`sided'")
						
						mata: decision=tableexact2(reg, nrows, ncols, ts, crit2s, "`sided'")
					
						drop _all
						getmata (reg)=reg, replace force
						local regflag=reg 
					
						if (`regflag'==1) {
									drop _all
									getmata (n m cval alpha sided reg predictn predictm)=crit2s, replace force
		
									gen invn=1/n
									gen n2=n^2
									gen invm=1/m
									gen m2=m^2
									gen alphahalf=alpha^0.5
									qui reg cval n m invn invm n2 m2 alpha alphahalf
									
									mata: betas = st_matrix("e(b)")
									mata: decision=getroot2(crit2s, betas, ts, "`sided'")
									
								}
								
						quietly drop _all
						getmata (decision)=decision, replace force
						return scalar reject=decision
			}	
end



mata:
	void function getoptim(todo,b,c1,c2,c3,crit,g,H)
	{
		crit=(c1+c2*b+c3*b^0.5)^2
	}
end


mata: 
	function invreg(x,c1,c2,c3) return(c1+c2*x+c3*x^0.5)
end

mata: 
	real scalar function getroot2(real matrix crit2s, real rowvector betas, real scalar ts, string sided)
		{ 
			real scalar c1, c2, c3, testroot, rc, pvalue, max_cval, min_cval
			
			pvalue=9.9999 
			
			npred=crit2s[1,7]
			mpred=crit2s[2,8]
			max_cval=colmax(crit2s[.,3])
			min_cval=colmin(crit2s[.,3])
			
			c1=betas[1,9] + betas[1,1]*npred + betas[1,2]*mpred + betas[1,3]*(1/npred) + betas[1,4]*(1/mpred) + betas[1,5]*(npred^2) + betas[1,6]*(mpred^2)  - ts
			c2=betas[1,7]
			c3=betas[1,8]
			
			rc=mm_root(pvalue=.,&invreg(),0,1,0,10000,c1,c2,c3)	
			pvalue=round(pvalue,10e-4)
			
			//pvalue, ts, min_cval, max_cval
			if (ts>max_cval)  pvalue=0
			if (ts<min_cval)  pvalue=1
			//pvalue
			
			if (pvalue<0.05) 	decision=1
			if (pvalue>=0.05)   decision=0
						
			return(decision)
			
		}
end



////////////////////// two sided two sample test //////////////////////

mata:
	real scalar function qqsim(real scalar xn, real scalar yn)
		{
			real colvector x, y, z 
			real scalar log_vnk
			
			x = sort(runiform(xn,1),1)
			y = sort(runiform(yn,1),1)
			z = sort((x\y),1)	
			
			log_vnk=log(elr(x,z)*elr(y,z))
			//log_vnk
			
			return(log_vnk)
		}
end


mata:
	real scalar function qqt0(real colvector x, real colvector y)
		{
			real colvector z
			real scalar xn, yn, teststat
			
			xn=rows(x)
			yn=rows(y)
						
			x=sort(x,1)
			y=sort(y,1)
			z=sort((x\y),1)
			
			teststat=log(elr(x,z)*elr(y,z))
			//teststat
			
			return(teststat)
		}
end


mata:
	real scalar function elr(real colvector q1, real colvector qq)
		{
			real colvector m, rm, mm, L, LL, U, UU, qL, qU, F
			real rowvector temp, prodFmat
			real matrix Fmat
			real scalar q1n, qqn, elr
			
			q1n=rows(q1)
			qqn=rows(qq)

			m=(round(q1n^0.6)::min(( round(q1n^.9),round(q1n/2) )))

			rm=m#J(q1n,1,1)
			mm=J(rows(m),1,1)#(1::q1n)

			L=mm-rm
			LL=(L:*(L:>0))+(L:<=0)
			U=mm+rm
			UU=(U:*(U:<=q1n))+(U:>q1n)*q1n

			qL=q1[LL]
			qU=q1[UU]
			
			F=mm_relrank(qq,rows(qq),qU)-mm_relrank(qq,rows(qq),qL)
			F=editvalue(F,0,1/(qqn))

			F=(2*rm):/(q1n*F)
						
			Fmat=colshape(F,q1n)
			_transpose(Fmat)
				
			prodFmat=exp(colsum(log(Fmat)))
			
			elr=min(prodFmat)
		
			return(elr)
		}
end

/////////////////////One sided two sample///////////////////////////

mata:
	real scalar function qqsim1s(real scalar xn, real scalar yn)
		{
			real colvector x, y 
			real scalar log_vnk
			
			x = sort(runiform(xn,1),1)
			y = sort(runiform(yn,1),1)
			
			log_vnk=log(elr1sx(x,y)*elr1sy(y,x))
			//log_vnk
			
			return(log_vnk)
		}
end


mata:
	real scalar function qqt01s(real colvector x, real colvector y)
		{
			real scalar xn, yn, teststat
			
			xn=rows(x)
			yn=rows(y)
						
			x=sort(x,1)
			y=sort(y,1)
			
			teststat=log(elr1sx(x,y)*elr1sy(y,x))
			//teststat
			
			return(teststat)
		}
end


mata:
	real scalar function elr1sx(real colvector q1, real colvector qq)
		{
		
			real colvector m, rm, mm, L, LL, U, UU, qL, qU, F
			real rowvector temp, prodFmat
			real matrix Fmat
			real scalar q1n, qqn, elr
			
			q1n=rows(q1)
			qqn=rows(qq)
			invqt=1/(q1n+qqn)

			m=(round(q1n^0.6)::min(( round(q1n^.9),round(q1n/2) )))

			rm=m#J(q1n,1,1)
			mm=J(rows(m),1,1)#(1::q1n)

			L=mm-rm
			LL=(L:*(L:>0))+(L:<=0)
			U=mm+rm
			UU=(U:*(U:<=q1n))+(U:>q1n)*q1n

			qL=q1[LL]
			qU=q1[UU]
			
		    F=rowmax((mm_relrank(q1,q1n,qU),mm_relrank(qq,qqn,qU)))-rowmax((mm_relrank(q1,q1n,qL),mm_relrank(qq,qqn,qL)))
					
			F=editvalue(F,0,invqt)

			F=(2*rm):/(q1n*F)
			Fmat=colshape(F,q1n)
			_transpose(Fmat)
			
			prodFmat=exp(colsum(log(Fmat)))
		
			elr=min(prodFmat)
	
			return(elr)
		}
end

mata:
	real scalar function elr1sy(real colvector q1, real colvector qq)
		{
		
			real colvector m, rm, mm, L, LL, U, UU, qL, qU, F
			real rowvector temp, prodFmat
			real matrix Fmat
			real scalar q1n, qqn, elr
			
			q1n=rows(q1)
			qqn=rows(qq)
			invqt=1/(q1n+qqn)

			m=(round(q1n^0.6)::min(( round(q1n^.9),round(q1n/2) )))

			rm=m#J(q1n,1,1)
			mm=J(rows(m),1,1)#(1::q1n)

			L=mm-rm
			LL=(L:*(L:>0))+(L:<=0)
			U=mm+rm
			UU=(U:*(U:<=q1n))+(U:>q1n)*q1n

			qL=q1[LL]
			qU=q1[UU]
			
		    F=rowmin((mm_relrank(q1,q1n,qU),mm_relrank(qq,qqn,qU)))-rowmin((mm_relrank(q1,q1n,qL),mm_relrank(qq,qqn,qL)))
			
			
			F=editvalue(F,0,invqt)

			F=(2*rm):/(q1n*F)
			Fmat=colshape(F,q1n)
			_transpose(Fmat)
			
			prodFmat=exp(colsum(log(Fmat)))
		
			elr=min(prodFmat)
	
			return(elr)
		}
end




////////////////////////// begin and branch by test ////////////////////////

//////////////// mc /////////////////
mata:
	void dbelmc(real matrix x, real scalar rnum, string dsnlist, string testside)
	{
		
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
		
		mc=J(rnum,1,0)
		
		//////// two sided one sample ////////
		if (strmatch(testside,"one.sample"))
			{
				nn=1
				yy=-xx
				while(nn<=rnum) {   
					  mc[nn,1]=wwsim(rows(xx),rows(yy))
					  ++nn
					}
				mc=sort(mc,1)
				t0=wwt0(xx,yy)
				pvalue=(colsum(mc:>=t0))/rnum
				
				test="DBEL (1-sample, 2-sided)"
				
				printf("{txt}{space 25}{c |}  T0               P-value\n")
				printf("{hline 25}{c +}{hline 34}\n")
				printf("{txt}%24s {c |} {res}%10.0g %10.0g\n",test, t0, pvalue)
				printf("{txt}{space 25}\n\n\n")
			}
			
		//////// one sided one sample ////////	
		if (strmatch(testside,"greater") & cols(x)==1)
			{
				nn=1
				yy=-xx
				while(nn<=rnum) {   
					  mc[nn,1]=wwsim1s(rows(xx),rows(yy))
					  ++nn
					}
				mc=sort(mc,1)
				t0=wwt01s(xx,yy)
				pvalue=(colsum(mc:>=t0))/rnum
				
				test="DBEL (1-sample, 1-sided)"
				
				printf("{txt}{space 25}{c |}  T0               P-value\n")
				printf("{hline 25}{c +}{hline 34}\n")
				printf("{txt}%24s {c |} {res}%10.0g %10.0g\n",test, t0, pvalue)
				printf("{txt}{space 25}\n\n\n")
			}	
	}
		
end

////////////// table /////////////////
mata:
real scalar reg( real matrix x)
	{
		nrows=J(1,3,0)
		reg=J(1,1,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		if (nrows[1,1]>30 | nrows[1,2]>30 | nrows[1,3]>30 )  reg=1
		
		return(reg)
	}		
end

mata:
real rowvector nrows( real matrix x)
	{
		nrows=J(1,3,0)
		if (cols(x)>=3) nrows[1,3]=colsum((x[.,3]:<.))
		if (cols(x)>=2) nrows[1,2]=colsum((x[.,2]:<.))
		if (cols(x)>=1) nrows[1,1]=colsum((x[.,1]:<.))
		
		nrows=nrows[1,1],nrows[1,2],nrows[1,3]
	
		return(nrows)
	}		
end


mata:
real scalar dbeltable2(real scalar reg, real rowvector nrows, real matrix crit2s, real matrix x, string dsnlist, string testside)
	{		
		if (cols(x)>=1) xx=select(x[.,1],x[.,1]:<.)
		if (cols(x)>=2) yy=select(x[.,2],x[.,2]:<.)
		if (cols(x)>=3) zz=select(x[.,3],x[.,3]:<.)
				
		//////////////////////////////////////////// two sample /////////////////////////////////////////////

		if (strmatch(testside,"greater") & cols(x)==2)
			{
				crit2s=select(crit2s[.,1::14],crit2s[.,5]:==1) //selects sided		
				t0=qqt01s(xx,yy)
				
				if (reg==1) {
								crit2s=select(crit2s[.,1::14],crit2s[.,1]:>30 :| crit2s[.,2]:>30)
								
								if ((30<nrows[1,1]  & nrows[1,1]<35)   |  (30<nrows[1,2]  & nrows[1,2]<35))		crit2s=select(crit2s[.,1::14],crit2s[.,6]:==1)
								if ((35<=nrows[1,1] & nrows[1,1]<40)   |  (35<=nrows[1,2] & nrows[1,2]<40))		crit2s=select(crit2s[.,1::14],crit2s[.,7]:==1)
								if ((40<=nrows[1,1] & nrows[1,1]<50)   |  (40<nrows[1,2]  & nrows[1,2]<50))		crit2s=select(crit2s[.,1::14],crit2s[.,8]:==1)
								if ((50<=nrows[1,1] & nrows[1,1]<60)   |  (50<nrows[1,2]  & nrows[1,2]<60))		crit2s=select(crit2s[.,1::14],crit2s[.,9]:==1)
								if ((60<=nrows[1,1] & nrows[1,1]<80)   |  (60<nrows[1,2]  & nrows[1,2]<80))		crit2s=select(crit2s[.,1::14],crit2s[.,10]:==1)
								if ((80<=nrows[1,1] & nrows[1,1]<100)  |  (80<nrows[1,2]  & nrows[1,2]<100))	crit2s=select(crit2s[.,1::14],crit2s[.,11]:==1)
								if ((100<=nrows[1,1] & nrows[1,1]<150) |  (100<nrows[1,2]  & nrows[1,2]<150))	crit2s=select(crit2s[.,1::14],crit2s[.,12]:==1)
								if ((150<=nrows[1,1] & nrows[1,1]<200) |  (1500<nrows[1,2] & nrows[1,2]<200))	crit2s=select(crit2s[.,1::14],crit2s[.,13]:==1)
								if (200<=nrows[1,1] | 200<=nrows[1,2]) 											crit2s=select(crit2s[.,1::14],crit2s[.,14]:==1)		
								
								if (30>=nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:==nrows[1,1])
								if (30>=nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:==nrows[1,2])
								if (30<nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:>30)
								if (30<nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:>30)
								
								crit2s=crit2s[.,1::5]
																					
								addinfo=J(rows(crit2s),3,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								addinfo[.,3]=addinfo[.,1]:*nrows[1,2]
								crit2s=(crit2s,addinfo)	
							}	
		
				if (reg==0) 	crit2s=select(crit2s[.,1::5],crit2s[.,1]:==nrows[1,1] :& crit2s[.,2]:==nrows[1,2])
									
			}

	
		if (strmatch(testside,"two.sample") & cols(x)==2)
			{
				crit2s=select(crit2s[.,1::14],crit2s[.,5]:==2)
				t0=qqt0(xx,yy)

				if (reg==1) {
								crit2s=select(crit2s[.,1::14],crit2s[.,1]:>30 :| crit2s[.,2]:>30)
								
								if ((30<nrows[1,1]  & nrows[1,1]<35)   |  (30<nrows[1,2]  & nrows[1,2]<35))		crit2s=select(crit2s[.,1::14],crit2s[.,6]:==1)
								if ((35<=nrows[1,1] & nrows[1,1]<40)   |  (35<=nrows[1,2] & nrows[1,2]<40))		crit2s=select(crit2s[.,1::14],crit2s[.,7]:==1)
								if ((40<=nrows[1,1] & nrows[1,1]<50)   |  (40<nrows[1,2]  & nrows[1,2]<50))		crit2s=select(crit2s[.,1::14],crit2s[.,8]:==1)
								if ((50<=nrows[1,1] & nrows[1,1]<60)   |  (50<nrows[1,2]  & nrows[1,2]<60))		crit2s=select(crit2s[.,1::14],crit2s[.,9]:==1)
								if ((60<=nrows[1,1] & nrows[1,1]<80)   |  (60<nrows[1,2]  & nrows[1,2]<80))		crit2s=select(crit2s[.,1::14],crit2s[.,10]:==1)
								if ((80<=nrows[1,1] & nrows[1,1]<100)  |  (80<nrows[1,2]  & nrows[1,2]<100))	crit2s=select(crit2s[.,1::14],crit2s[.,11]:==1)
								if ((100<=nrows[1,1] & nrows[1,1]<150) |  (100<nrows[1,2]  & nrows[1,2]<150))	crit2s=select(crit2s[.,1::14],crit2s[.,12]:==1)
								if ((150<=nrows[1,1] & nrows[1,1]<200) |  (1500<nrows[1,2] & nrows[1,2]<200))	crit2s=select(crit2s[.,1::14],crit2s[.,13]:==1)
								if (200<=nrows[1,1] | 200<=nrows[1,2]) 											crit2s=select(crit2s[.,1::14],crit2s[.,14]:==1)		
								
								if (30>=nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:==nrows[1,1])
								if (30>=nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:==nrows[1,2])
								if (30<nrows[1,1])	crit2s=select(crit2s[.,1::14],crit2s[.,1]:>30)
								if (30<nrows[1,2])	crit2s=select(crit2s[.,1::14],crit2s[.,2]:>30)
								
								crit2s=crit2s[.,1::5]
																							
								addinfo=J(rows(crit2s),3,1)
								addinfo[.,1]=addinfo[.,1]:*reg[1,1]
								addinfo[.,2]=addinfo[.,1]:*nrows[1,1]
								addinfo[.,3]=addinfo[.,1]:*nrows[1,2]
								crit2s=(crit2s,addinfo)	
							}
							
				if (reg==0) 	crit2s=select(crit2s[.,1::5],crit2s[.,1]:==nrows[1,1] :& crit2s[.,2]:==nrows[1,2])
		
			}
			
		return(t0)
	}
		
end


mata:

real scalar tableexact2(real scalar reg,real rowvector nrows, real scalar ncols, real scalar ts, real matrix crit2s, string testside)
	{
		real scalar decision
		decision=.
		
		//////// two sample -- 1s & 2s  ////////
		if (ncols==2 & reg==0)
			{
				Tcrit2s=crit2s'	
				if (ts>Tcrit2s[3,5]) 	decision=1
				if (Tcrit2s[3,5]>=ts) 	decision=0				
			}
		return(decision)
	}
		
end
	
	
//vx_dbel dsn1, sided("greater") method("table")
//vx_dbel dsn1, sided("greater") method("mc")

//vx_dbel dsn1 , sided("one.sample") method("table")
//vx_dbel dsn1, sided("one.sample") method("mc")


	//generate test data to analyze
	//mata: dsn1 = runiform(16,1):-0.5
	//mata: dsn1 = rgamma(34, 1, 9, 0.5):-7
	//mata: dsn1 = rnormal(91, 1, 0, 1):+0.2
	//mata: dsn1 = rgamma(34, 1, 9, 0.5):-7
	
	
//////////////////////////////// 5 ////////////////////////////	
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(5, 1, 1, 1)
	mata: dsn2 = rnormal(5, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera1=colsum(result)/10000
mata: powera1	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(5, 1, 1, 1)
	mata: dsn2 = rnormal(10, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera2=colsum(result)/10000
mata: powera2	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(5, 1, 1, 1)
	mata: dsn2 = rnormal(15, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera3=colsum(result)/10000
mata: powera3	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(5, 1, 1, 1)
	mata: dsn2 = rnormal(20, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera4=colsum(result)/10000
mata: powera4	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(5, 1, 1, 1)
	mata: dsn2 = rnormal(25, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera5=colsum(result)/10000
mata: powera5	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(5, 1, 1, 1)
	mata: dsn2 = rnormal(30, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera6=colsum(result)/10000
mata: powera6	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(5, 1, 1, 1)
	mata: dsn2 = rnormal(35, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera7=colsum(result)/10000
mata: powera7	

mata: powera1,powera2,powera3,powera4,powera5,powera6,powera7


//////////////////////////////// 10 ////////////////////////////	
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(10, 1, 1, 1)
	mata: dsn2 = rnormal(5, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera1=colsum(result)/10000
mata: powera1	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(10, 1, 1, 1)
	mata: dsn2 = rnormal(10, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera2=colsum(result)/10000
mata: powera2	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(10, 1, 1, 1)
	mata: dsn2 = rnormal(15, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera3=colsum(result)/10000
mata: powera3	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(10, 1, 1, 1)
	mata: dsn2 = rnormal(20, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera4=colsum(result)/10000
mata: powera4	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(10, 1, 1, 1)
	mata: dsn2 = rnormal(25, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera5=colsum(result)/10000
mata: powera5	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(10, 1, 1, 1)
	mata: dsn2 = rnormal(30, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera6=colsum(result)/10000
mata: powera6	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(10, 1, 1, 1)
	mata: dsn2 = rnormal(35, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera7=colsum(result)/10000
mata: powera7	

mata: powera1,powera2,powera3,powera4,powera5,powera6,powera7



//////////////////////////////// 15 ////////////////////////////	
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(15, 1, 1, 1)
	mata: dsn2 = rnormal(5, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera1=colsum(result)/10000
mata: powera1	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(15, 1, 1, 1)
	mata: dsn2 = rnormal(10, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera2=colsum(result)/10000
mata: powera2	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(15, 1, 1, 1)
	mata: dsn2 = rnormal(15, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera3=colsum(result)/10000
mata: powera3	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(15, 1, 1, 1)
	mata: dsn2 = rnormal(20, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera4=colsum(result)/10000
mata: powera4	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(15, 1, 1, 1)
	mata: dsn2 = rnormal(25, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera5=colsum(result)/10000
mata: powera5	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(15, 1, 1, 1)
	mata: dsn2 = rnormal(30, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera6=colsum(result)/10000
mata: powera6	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(15, 1, 1, 1)
	mata: dsn2 = rnormal(35, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera7=colsum(result)/10000
mata: powera7	

mata: powera1,powera2,powera3,powera4,powera5,powera6,powera7





//////////////////////////////// 20 ////////////////////////////	
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(20, 1, 1, 1)
	mata: dsn2 = rnormal(5, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera1=colsum(result)/10000
mata: powera1	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(20, 1, 1, 1)
	mata: dsn2 = rnormal(10, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera2=colsum(result)/10000
mata: powera2	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(20, 1, 1, 1)
	mata: dsn2 = rnormal(15, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera3=colsum(result)/10000
mata: powera3	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(20, 1, 1, 1)
	mata: dsn2 = rnormal(20, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera4=colsum(result)/10000
mata: powera4	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(20, 1, 1, 1)
	mata: dsn2 = rnormal(25, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera5=colsum(result)/10000
mata: powera5	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(20, 1, 1, 1)
	mata: dsn2 = rnormal(30, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera6=colsum(result)/10000
mata: powera6	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(20, 1, 1, 1)
	mata: dsn2 = rnormal(35, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera7=colsum(result)/10000
mata: powera7	

mata: powera1,powera2,powera3,powera4,powera5,powera6,powera7




//////////////////////////////// 30 ////////////////////////////	
mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(30, 1, 1, 1)
	mata: dsn2 = rnormal(5, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera1=colsum(result)/10000
mata: powera1	
	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(30, 1, 1, 1)
	mata: dsn2 = rnormal(10, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera2=colsum(result)/10000
mata: powera2	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(30, 1, 1, 1)
	mata: dsn2 = rnormal(15, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera3=colsum(result)/10000
mata: powera3	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(30, 1, 1, 1)
	mata: dsn2 = rnormal(20, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera4=colsum(result)/10000
mata: powera4	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(30, 1, 1, 1)
	mata: dsn2 = rnormal(25, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera5=colsum(result)/10000
mata: powera5	

mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(30, 1, 1, 1)
	mata: dsn2 = rnormal(30, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera6=colsum(result)/10000
mata: powera6	


mata: result=J(10000,1,9)									
forvalues i = 1(1)10000 {	
	//generate test data to analyze
	quietly drop _all
	mata: dsn1 = rnormal(30, 1, 1, 1)
	mata: dsn2 = rnormal(35, 1, 0,0.5)

	getmata dsn1 dsn2
	
	
	//vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	vx_dbel dsn1 dsn2, sided("two.sample") method("reg")
	
	generate decisioni=r(reject)
	generate numi=`i'
	quietly keep in 1
    quietly putmata decision=decisioni i=numi, replace omitmissing
	//mata: i,decision
	mata: 	result[i,1]=decision
}

		
mata: powera7=colsum(result)/10000
mata: powera7	

mata: powera1,powera2,powera3,powera4,powera5,powera6,powera7

